from py_rejseplan.enums import TransportClass

# Mapping from string keys to TransportClass enum values
DEPARTURE_TYPE_TO_CLASS = {
    "ic": TransportClass.IC,
    "icl": TransportClass.ICL,
    "re": TransportClass.RE,
    "tog": TransportClass.TOG,
    "s_tog": TransportClass.S_TOG,
    "bus": TransportClass.BUS,
    "express_bus": TransportClass.EXPRESS_BUS,
    "night_bus": TransportClass.NIGHT_BUS,
    "flexible_bus": TransportClass.FLEXIBLE_BUS,
    "ferry": TransportClass.FERRY,
    "metro": TransportClass.METRO,
    "letbane": TransportClass.LETBANE,
    "flight": TransportClass.FLIGHT,
}

CATOUT_TO_CLASS = {
    # IC Group (cls=1)
    "IC": TransportClass.IC,
    "IB": TransportClass.IC,
    # ICL Group (cls=2)
    "ICL": TransportClass.ICL,
    "ICL-X": TransportClass.ICL,
    "IL": TransportClass.ICL,  # ICL+
    # Regional trains (cls=4)
    "Re": TransportClass.RE,
    "RA": TransportClass.RE,
    "RX": TransportClass.RE,
    # Long distance trains (cls=8)
    "EC": TransportClass.TOG,
    "IR": TransportClass.TOG,
    "IP": TransportClass.TOG,
    "ICE": TransportClass.TOG,
    "SJ": TransportClass.TOG,
    "EN": TransportClass.TOG,
    "ICN": TransportClass.TOG,
    "Pågatog": TransportClass.TOG,
    "NAT": TransportClass.TOG,  # Night trains
    "L": TransportClass.TOG,  # Local trains
    "SKOLE": TransportClass.TOG,
    "MTOG": TransportClass.TOG,
    "E-Tog": TransportClass.TOG,
    "R-netTog": TransportClass.TOG,
    # S-trains (cls=16)
    "S-Tog": TransportClass.S_TOG,
    # Regular buses (cls=32)
    "Bus": TransportClass.BUS,
    "Bybus": TransportClass.BUS,
    "E-Bus": TransportClass.BUS,
    "ServiceB": TransportClass.BUS,
    "R-net": TransportClass.BUS,
    "C-Bus": TransportClass.BUS,
    "TraktBus": TransportClass.BUS,
    "Taxa": TransportClass.BUS,
    # Express buses (cls=64)
    "X Bus": TransportClass.EXPRESS_BUS,
    "Ekspresb": TransportClass.EXPRESS_BUS,
    "Fjernbus": TransportClass.EXPRESS_BUS,
    # Night and special buses (cls=128)
    "Natbus": TransportClass.NIGHT_BUS,
    "Havnebus": TransportClass.NIGHT_BUS,
    "Flybus": TransportClass.NIGHT_BUS,
    "Sightseeing bus": TransportClass.NIGHT_BUS,
    "HV-bus": TransportClass.NIGHT_BUS,
    "Si-bus": TransportClass.NIGHT_BUS,
    # Flexible transport (cls=256)
    "Flexbus": TransportClass.FLEXIBLE_BUS,
    "Flextur": TransportClass.FLEXIBLE_BUS,
    "TELEBUS": TransportClass.FLEXIBLE_BUS,
    "Nærbus": TransportClass.FLEXIBLE_BUS,
    # Ferry (cls=512)
    "Færge": TransportClass.FERRY,
    "HF": TransportClass.FERRY,  # Fast ferry
    # Metro (cls=1024)
    "MET": TransportClass.METRO,
    # Light rail (cls=2048)
    "Letbane": TransportClass.LETBANE,
    "LTBUS": TransportClass.LETBANE,
    # Flight (cls=4096)
    "Fly": TransportClass.FLIGHT,
}


from enum import IntEnum

class TransportClass(IntEnum):
    """Transport class numbers from Rejseplanen XML."""

    IC = 1  # InterCity trains
    ICL = 2  # InterCity Lyn trains
    RE = 4  # Regional trains
    TOG = 8  # Long distance trains
    S_TOG = 16  # S-trains (Copenhagen suburban)
    BUS = 32  # Regular city buses
    EXPRESS_BUS = 64  # Express/long-distance buses (S-bus/E-bus)
    NIGHT_BUS = 128  # Night buses (N-bus)
    FLEXIBLE_BUS = 256  # Flexible transport (Divbus)
    FERRY = 512  # Ferry
    METRO = 1024  # Metro
    LETBANE = 2048  # Light rail
    FLIGHT = 4096  # Flight


from typing import Optional
from py_rejseplan.enums import TransportClass
from py_rejseplan.dataclasses.transport_mappings import CATOUT_TO_CLASS

class TransportClassMixin:
    """Mixin to add transport class functionality to dataclasses."""

    cls: Optional[int] = None  # Placeholder for the cls attribute
    catOut: Optional[str] = None  # Placeholder for the catOut attribute
    
    def get_transport_class(self) -> Optional[TransportClass]:
        """Get the transport class based on the catOut attribute.
        Returns:
            Optional[TransportClass]: The transport class if found, else None.
        """
        if self.cls is not None:
            try:
                return TransportClass(self.cls)
            except ValueError:
                pass  # Invalid cls value, fall back to catOut

        if self.catOut:
            transport_class = CATOUT_TO_CLASS.get(self.catOut)
            if transport_class:
                return transport_class

        return None


import pytest

from py_rejseplan.dataclasses.mixins import TransportClassMixin
from py_rejseplan.enums import TransportClass
from py_rejseplan.dataclasses.transport_mappings import DEPARTURE_TYPE_TO_CLASS, CATOUT_TO_CLASS

@pytest.mark.parametrize("transport_type,expected_class", [
    (TransportClass.BUS, "bus"),
    (TransportClass.IC, "IC"),
    (TransportClass.METRO, "metro"),
    (TransportClass.ICL, "ICL"),
])
def test_transport_class_mixin_get_transport_class_int(transport_type: TransportClass, expected_class: str):
    """Test TransportClassMixin.get_transport_class returns correct class for known transport types."""
    mixin = TransportClassMixin()
    mixin.cls = transport_type.value
    assert mixin.get_transport_class() == transport_type

@pytest.mark.parametrize("transport_type,expected_class", [
    ("EC", TransportClass.TOG),
    ("IR", TransportClass.TOG),
    ("IP", TransportClass.TOG),
    ("Bus", TransportClass.BUS),
    ("Bybus", TransportClass.BUS),
    ("E-Bus", TransportClass.BUS),
    ("ServiceB", TransportClass.BUS),
    ("R-net", TransportClass.BUS),
])
def test_transport_class_mixin_get_transport_class_str(transport_type: str, expected_class: TransportClass):
    """Test TransportClassMixin.get_transport_class returns correct class for known transport types."""
    mixin = TransportClassMixin()
    mixin.catOut = transport_type 
    assert mixin.get_transport_class() == expected_class

@pytest.mark.parametrize("transport_type", [
    "spaceship",
    "",
    None,
])
def test_transport_class_mixin_get_transport_class_unknown(transport_type: str):
    """Test TransportClassMixin.get_transport_class returns 'unknown' for unknown transport types."""
    mixin = TransportClassMixin()
    mixin.catOut = transport_type
    assert mixin.get_transport_class() is None

